@@ -2,7 +2,7 @@
 #
 #       clipboard.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -87,27 +87,32 @@ class ClipboardHandler:
     def copy(self, text_view, from_codebox):
         """Copy to Clipboard"""
         text_buffer = text_view.get_buffer()
-        if not text_buffer.get_has_selection(): return
-        iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
-        num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
-        if from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
-            if num_chars > 30000: return
-        text_view.stop_emission("copy-clipboard")
-        self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
+        if text_buffer.get_has_selection():
+            iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
+            num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
+            override_copy = True
+            if (from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID) and num_chars > 30000:
+                print "copy-clipboard not overridden for num_chars", num_chars
+            else:
+                text_view.stop_emission("copy-clipboard")
+                self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
+        self.force_plain_text = False
 
     def cut(self, text_view, from_codebox):
         """Cut to Clipboard"""
         text_buffer = text_view.get_buffer()
-        if not text_buffer.get_has_selection(): return
-        iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
-        num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
-        if from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
-            if num_chars > 30000: return
-        text_view.stop_emission("cut-clipboard")
-        self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
-        if self.dad.is_curr_node_not_read_only_or_error():
-            text_buffer.delete_selection(True, text_view.get_editable())
-            text_view.grab_focus()
+        if text_buffer.get_has_selection():
+            iter_sel_start, iter_sel_end = text_buffer.get_selection_bounds()
+            num_chars = iter_sel_end.get_offset() - iter_sel_start.get_offset()
+            if (from_codebox or self.dad.syntax_highlighting != cons.RICH_TEXT_ID) and num_chars > 30000:
+                print "cut-clipboard not overridden for num_chars", num_chars
+            else:
+                text_view.stop_emission("cut-clipboard")
+                self.selection_to_clipboard(text_buffer, text_view, iter_sel_start, iter_sel_end, num_chars, from_codebox)
+                if self.dad.is_curr_node_not_read_only_or_error():
+                    text_buffer.delete_selection(True, text_view.get_editable())
+                    text_view.grab_focus()
+        self.force_plain_text = False
 
     def table_row_to_clipboard(self, table_dict):
         """Put the Selected Table Row to the Clipboard"""
@@ -160,15 +165,23 @@ class ClipboardHandler:
             text_offsets_range = [iter_sel_start.get_offset(), iter_sel_end.get_offset()]
             plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=text_offsets_range, check_link_target=True)
             rich_text = self.rich_text_get_from_text_buffer_selection(text_buffer, iter_sel_start, iter_sel_end)
-            targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0], TARGETS_HTML[1]]
-            if pixbuf_target: targets_vector.append(TARGETS_IMAGES[0])
+            if not self.force_plain_text:
+                targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0], TARGETS_HTML[1]]
+                if pixbuf_target:
+                    targets_vector.append(TARGETS_IMAGES[0])
+            else:
+                targets_vector = [TARGET_CTD_PLAIN_TEXT]
             self.clipboard.set_with_data([(t, 0, 0) for t in targets_vector],
                 self.get_func,
                 self.clear_func,
                 (plain_text, rich_text, html_text, pixbuf_target))
         else:
             plain_text = text_buffer.get_text(iter_sel_start, iter_sel_end)
-            self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGETS_HTML[0], TARGETS_HTML[1])],
+            if not self.force_plain_text:
+                targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGETS_HTML[0], TARGETS_HTML[1]]
+            else:
+                targets_vector = [TARGET_CTD_PLAIN_TEXT]
+            self.clipboard.set_with_data([(t, 0, 0) for t in targets_vector],
                                          self.get_func,
                                          self.clear_func,
                                          (plain_text, None, html_text, pixbuf_target))
@@ -210,11 +223,11 @@ class ClipboardHandler:
         if not targets: return
         self.dad.curr_buffer.delete_selection(True, sourceview.get_editable())
         if self.force_plain_text:
-            self.force_plain_text = False
             for target in TARGETS_PLAIN_TEXT:
                 if target in targets:
                     self.clipboard.request_contents(target, self.to_plain_text)
                     return
+            self.force_plain_text = False
         #print targets
         if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             if TARGET_CTD_RICH_TEXT in targets:
@@ -257,7 +270,7 @@ class ClipboardHandler:
         for element in uri_list:
             if not element: continue
             iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
-            if (element.startswith("http") or element.startswith("ftp")):
+            if support.get_first_chars_of_string_are(element, cons.WEB_LINK_STARTERS):
                 property_value = "webs " + element
             elif element.startswith("file://"):
                 file_path = element[7:].replace("%20", cons.CHAR_SPACE)
@@ -316,7 +329,7 @@ class ClipboardHandler:
         iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
         start_offset = iter_insert.get_offset()
         self.dad.curr_buffer.insert(iter_insert, plain_text)
-        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
+        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID and not self.force_plain_text:
             web_links_offsets = imports.get_web_links_offsets_from_plain_text(plain_text)
             if web_links_offsets:
                 for offsets in web_links_offsets:
@@ -341,6 +354,7 @@ class ClipboardHandler:
                         iter_sel_start.backward_chars(len(plain_text))
                         self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create("link", property_value),
                                                                iter_sel_start, iter_sel_end)
+        self.force_plain_text = False
 
     def to_rich_text(self, clipboard, selectiondata, data):
         """From Clipboard to Rich Text"""
