@@ -2,7 +2,7 @@
 #
 #       codeboxes.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ class CodeBoxesHandler:
         self.curr_codebox_anchor = None
         self.curr_v = 0
         self.curr_h = 0
+        self.key_down = False
 
     def codebox_in_use(self):
         """Returns a CodeBox SourceView if Currently in Use or None"""
@@ -251,8 +252,8 @@ class CodeBoxesHandler:
             anchor.sourcebuffer.set_modified(False)
         anchor.sourceview = gtksourceview2.View(anchor.sourcebuffer)
         anchor.sourceview.set_smart_home_end(gtksourceview2.SMART_HOME_END_AFTER)
-        if self.dad.highl_curr_line: anchor.sourceview.set_highlight_current_line(True)
-        if self.dad.show_white_spaces: anchor.sourceview.set_draw_spaces(DRAW_SPACES_FLAGS)
+        anchor.sourceview.set_highlight_current_line(self.dad.pt_highl_curr_line)
+        if self.dad.pt_show_white_spaces: anchor.sourceview.set_draw_spaces(DRAW_SPACES_FLAGS)
         if anchor.syntax_highlighting == cons.PLAIN_TEXT_ID:
             anchor.sourceview.modify_font(pango.FontDescription(self.dad.text_font))
         else:
@@ -260,9 +261,9 @@ class CodeBoxesHandler:
         anchor.sourceview.set_show_line_numbers(anchor.show_line_numbers)
         anchor.sourceview.set_insert_spaces_instead_of_tabs(self.dad.spaces_instead_tabs)
         anchor.sourceview.set_tab_width(self.dad.tabs_width)
-        anchor.sourceview.set_auto_indent(self.dad.auto_indent)
         anchor.sourceview.connect('populate-popup', self.on_sourceview_populate_popup_codebox, anchor)
         anchor.sourceview.connect('key_press_event', self.on_key_press_sourceview_codebox, anchor)
+        anchor.sourceview.connect('key_release_event', self.on_key_release_sourceview_codebox, anchor)
         anchor.sourceview.connect('button-press-event', self.on_mouse_button_clicked_codebox, anchor)
         anchor.sourceview.connect("event-after", self.on_sourceview_event_after_codebox, anchor)
         anchor.sourceview.connect("motion-notify-event", self.on_sourceview_motion_notify_event_codebox)
@@ -291,7 +292,7 @@ class CodeBoxesHandler:
             self.dad.state_machine.apply_object_justification(text_iter, codebox_justification, text_buffer)
         elif self.dad.user_active:
             # if I apply a justification, the state is already updated
-            self.dad.state_machine.update_state(self.dad.treestore[self.dad.curr_tree_iter][3])
+            self.dad.state_machine.update_state()
 
     def codebox_increase_width(self, *args):
         """Increase CodeBox Width"""
@@ -427,11 +428,18 @@ class CodeBoxesHandler:
                 if not self.dad.codebox_sentinel_id: self.dad.codebox_sentinel_start()
         return False
 
+    def on_key_release_sourceview_codebox(self, widget, event, anchor):
+        """Catches CodeBox Key Releases"""
+        self.key_down = False
+        return False
+
     def on_key_press_sourceview_codebox(self, widget, event, anchor):
-        """Extend the Default Right-Click Menu of the CodeBox"""
-        if not self.dad.user_active: return False
+        """Catches CodeBox Key Presses"""
+        self.key_down = True
+        if not self.dad.user_active:
+            return False
+        keyname = gtk.gdk.keyval_name(event.keyval)
         if event.state & gtk.gdk.CONTROL_MASK:
-            keyname = gtk.gdk.keyval_name(event.keyval)
             self.curr_codebox_anchor = anchor
             if keyname == "period":
                 if event.state & gtk.gdk.MOD1_MASK:
@@ -441,6 +449,23 @@ class CodeBoxesHandler:
                 if event.state & gtk.gdk.MOD1_MASK:
                     self.codebox_decrease_height()
                 else: self.codebox_increase_height()
+        elif (event.state & gtk.gdk.SHIFT_MASK):
+            if keyname == cons.STR_KEY_SHIFT_TAB:
+                text_buffer = widget.get_buffer()
+                if not text_buffer.get_has_selection():
+                    iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+                    list_info = self.dad.lists_handler.get_paragraph_list_info(iter_insert)
+                    if list_info and list_info["level"]:
+                        support.on_sourceview_list_change_level(self.dad, iter_insert, list_info, text_buffer, False)
+                        return True
+        elif keyname == cons.STR_KEY_TAB:
+            text_buffer = widget.get_buffer()
+            if not text_buffer.get_has_selection():
+                iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+                list_info = self.dad.lists_handler.get_paragraph_list_info(iter_insert)
+                if list_info:
+                    support.on_sourceview_list_change_level(self.dad, iter_insert, list_info, text_buffer, True)
+                    return True
         return False
 
     def on_mouse_button_clicked_codebox(self, widget, event, anchor):
