@@ -2,7 +2,7 @@
 #
 #       config.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -33,8 +33,9 @@ COLOR_PALETTE_DEFAULT = ["#000000", "#ffffff", "#7f7f7f", "#ff0000", "#a020f0",
                          "#0000ff", "#add8e6", "#00ff00", "#ffff00", "#ffa500",
                          "#e6e6fa", "#a52a2a", "#8b6914", "#1e90ff", "#ffc0cb",
                          "#90ee90", "#1a1a1a", "#4d4d4d", "#bfbfbf", "#e5e5e5"]
-SPECIAL_CHARS_DEFAULT = "“”„•◇▪▸☐☑☒…‰€©®™°↓↑→←↔↵⇓⇑⇒⇐⇔»«▼▲►◄≤≥≠±¹²³½¼⅛×÷∞ø∑√∫ΔδΠπΣΦΩωαβγεηλμ☺☻☼♥♀♂♪♫"
+SPECIAL_CHARS_DEFAULT = "“”„•◇▪▸☐☑☒★…‰€©®™°↓↑→←↔↵⇓⇑⇒⇐⇔»«▼▲►◄≤≥≠≈±¹²³½¼⅛×÷∞ø∑√∫ΔδΠπΣΦΩωαβγεηλμ☺☻☼♥♣♦✔♀♂♪♫✝"
 SELWORD_CHARS_DEFAULT = ".-@"
+TIMESTAMP_FORMAT_DEFAULT = "%Y/%m/%d - %H:%M"
 TOOLBAR_VEC_DEFAULT = ["TreeAddNode", "TreeAddSubNode", cons.TAG_SEPARATOR, "GoBack", "GoForward", cons.TAG_SEPARATOR, cons.CHAR_STAR, "Save", "Export2PDF", cons.TAG_SEPARATOR, "FindInNodes", cons.TAG_SEPARATOR, "BulletedList", "NumberedList", "ToDoList", cons.TAG_SEPARATOR, "HandleImage", "HandleTable", "HandleCodeBox", "EmbFileInsert", "HandleLink", "HandleAnchor", cons.TAG_SEPARATOR, "RemoveFormatting", "ColorForeground", "ColorBackground", "Bold", "Italic", "Underline", "Strikethrough", "H1", "H2", "H3", "Small", "Superscript", "Subscript", "Monospace"]
 TOOLBAR_VEC_BLACKLIST = ["CutAnchor", "CopyAnchor", "DeleteAnchor", "EditAnchor", "CutEmbFile", "CopyEmbFile", "DeleteEmbFile", "EmbFileSave", "EmbFileOpen", "SaveImage", "EditImage", "CutImage", "CopyImage", "DeleteImage", "EditImageLink", "DismissImageLink", "ShowHideMainWin"]
 SEPARATOR_ASCII_REPR = "---------"
@@ -139,6 +140,7 @@ def config_file_load(inst):
         inst.expcollstr3 = config.get(section, "expcollstr3") if config.has_option(section, "expcollstr3") else ""
         inst.expcollsel3 = config.get(section, "expcollsel3") if config.has_option(section, "expcollsel3") else ""
         inst.expcollcur3 = config.getint(section, "expcollcur3") if config.has_option(section, "expcollcur3") else 0
+        inst.nodes_bookm_exp = config.getboolean(section, "nodes_bookm_exp") if config.has_option(section, "nodes_bookm_exp") else True
         inst.nodes_icons = config.get(section, "nodes_icons") if config.has_option(section, "nodes_icons") else "c"
         inst.tree_right_side = config.getboolean(section, "tree_right_side") if config.has_option(section, "tree_right_side") else False
         inst.cherry_wrap_width = config.getint(section, "cherry_wrap_width") if config.has_option(section, "cherry_wrap_width") else 130
@@ -158,12 +160,13 @@ def config_file_load(inst):
         inst.wrapping_indent = config.getint(section, "wrapping_indent") if config.has_option(section, "wrapping_indent") else -14
         inst.auto_indent = config.getboolean(section, "auto_indent") if config.has_option(section, "auto_indent") else True
         inst.rt_show_white_spaces = config.getboolean(section, "rt_show_white_spaces") if config.has_option(section, "rt_show_white_spaces") else False
-        inst.show_white_spaces = config.getboolean(section, "show_white_spaces") if config.has_option(section, "show_white_spaces") else True
-        inst.highl_curr_line = config.getboolean(section, "highl_curr_line") if config.has_option(section, "highl_curr_line") else True
+        inst.pt_show_white_spaces = config.getboolean(section, "pt_show_white_spaces") if config.has_option(section, "pt_show_white_spaces") else True
+        inst.rt_highl_curr_line = config.getboolean(section, "rt_highl_curr_line") if config.has_option(section, "rt_highl_curr_line") else True
+        inst.pt_highl_curr_line = config.getboolean(section, "pt_highl_curr_line") if config.has_option(section, "pt_highl_curr_line") else True
         inst.h_rule = config.get(section, "h_rule") if config.has_option(section, "h_rule") else HORIZONTAL_RULE
         inst.special_chars = unicode(config.get(section, "special_chars") if config.has_option(section, "special_chars") else SPECIAL_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
         inst.selword_chars = unicode(config.get(section, "selword_chars") if config.has_option(section, "selword_chars") else SELWORD_CHARS_DEFAULT, cons.STR_UTF8, cons.STR_IGNORE)
-        inst.timestamp_format = config.get(section, "timestamp_format") if config.has_option(section, "timestamp_format") else "%Y/%m/%d - %H:%M"
+        inst.timestamp_format = config.get(section, "timestamp_format") if config.has_option(section, "timestamp_format") else TIMESTAMP_FORMAT_DEFAULT
         inst.links_relative = config.getboolean(section, "links_relative") if config.has_option(section, "links_relative") else False
         if config.has_option(section, "weblink_custom_action"):
             temp_str = config.get(section, "weblink_custom_action")
@@ -291,7 +294,7 @@ def config_file_load(inst):
         inst.weblink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_WEB]
         inst.filelink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_FILE]
         inst.folderlink_custom_action = [False, LINK_CUSTOM_ACTION_DEFAULT_FILE]
-        inst.timestamp_format = "%Y/%m/%d - %H:%M"
+        inst.timestamp_format = TIMESTAMP_FORMAT_DEFAULT
         inst.codebox_width = 700
         inst.codebox_height = 100
         inst.codebox_width_pixels = True
@@ -306,9 +309,11 @@ def config_file_load(inst):
         inst.backup_num = 3
         inst.autosave_on_quit = False
         inst.tree_right_side = False
+        inst.nodes_bookm_exp = True
         inst.rt_show_white_spaces = False
-        inst.show_white_spaces = True
-        inst.highl_curr_line = True
+        inst.pt_show_white_spaces = True
+        inst.rt_highl_curr_line = True
+        inst.pt_highl_curr_line = True
         inst.hpaned_pos = 170
         inst.show_node_name_label = True
         inst.nodes_icons = "c"
@@ -319,7 +324,7 @@ def config_file_load(inst):
 def config_file_apply(inst):
     """Apply the Preferences from Config File"""
     inst.hpaned.set_property('position', inst.hpaned_pos)
-    inst.header_node_name_label.set_property(cons.STR_VISIBLE, inst.show_node_name_label)
+    inst.header_node_name_hbox.set_property(cons.STR_VISIBLE, inst.show_node_name_label)
     inst.set_treeview_font()
     inst.widget_set_colors(inst.treeview, inst.tt_def_fg, inst.tt_def_bg, False)
     if not pgsc_spellcheck.HAS_PYENCHANT:
@@ -338,6 +343,9 @@ def config_file_apply(inst):
     inst.ui.get_widget("/ToolBar").set_property("icon-size", ICONS_SIZE[inst.toolbar_icon_size])
     if inst.autosave[0]: inst.autosave_timer_start()
     if inst.enable_mod_time_sentinel: inst.modification_time_sentinel_start()
+    inst.aux_renderer_pixbuf.set_property("visible", len(inst.bookmarks) > 0)
+    inst.progresstop.hide()
+    inst.progressbar.hide()
 
 def config_file_save(inst):
     """Save the Preferences to Config File"""
@@ -396,6 +404,7 @@ def config_file_save(inst):
         config.set(section, "expcollstr3", inst.expcollstr3)
         config.set(section, "expcollsel3", inst.expcollsel3)
         config.set(section, "expcollcur3", inst.expcollcur3)
+    config.set(section, "nodes_bookm_exp", inst.nodes_bookm_exp)
     config.set(section, "nodes_icons", inst.nodes_icons)
     config.set(section, "tree_right_side", inst.tree_right_side)
     config.set(section, "cherry_wrap_width", inst.cherry_wrap_width)
@@ -416,8 +425,9 @@ def config_file_save(inst):
     config.set(section, "wrapping_indent", inst.wrapping_indent)
     config.set(section, "auto_indent", inst.auto_indent)
     config.set(section, "rt_show_white_spaces", inst.rt_show_white_spaces)
-    config.set(section, "show_white_spaces", inst.show_white_spaces)
-    config.set(section, "highl_curr_line", inst.highl_curr_line)
+    config.set(section, "pt_show_white_spaces", inst.pt_show_white_spaces)
+    config.set(section, "rt_highl_curr_line", inst.rt_highl_curr_line)
+    config.set(section, "pt_highl_curr_line", inst.pt_highl_curr_line)
     config.set(section, "h_rule", inst.h_rule)
     config.set(section, "special_chars", inst.special_chars)
     config.set(section, "selword_chars", inst.selword_chars)
@@ -508,25 +518,27 @@ def set_tree_expanded_collapsed_string(inst, treeview=None):
     if not treeview: treeview = inst.treeview
     treestore = treeview.get_model()
     treeview.collapse_all()
-    if not inst.expanded_collapsed_string: return
     expanded_collapsed_dict = {}
     expanded_collapsed_vector = inst.expanded_collapsed_string.split('_')
     for element in expanded_collapsed_vector:
-        couple = element.split(',')
-        expanded_collapsed_dict[couple[0]] = couple[1]
+        if "," in element:
+            couple = element.split(',')
+            expanded_collapsed_dict[couple[0]] = couple[1]
     tree_iter = treestore.get_iter_first()
     while tree_iter != None:
-        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore)
+        set_tree_expanded_collapsed_string_iter(inst, tree_iter, expanded_collapsed_dict, treeview, treestore)
         tree_iter = treestore.iter_next(tree_iter)
 
-def set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore):
+def set_tree_expanded_collapsed_string_iter(inst, tree_iter, expanded_collapsed_dict, treeview, treestore):
     """Iter of the Expanded and Collapsed Nodes Parsing"""
     node_id = str(treestore[tree_iter][3])
     if node_id in expanded_collapsed_dict and expanded_collapsed_dict[node_id] == "True":
         treeview.expand_row(treestore.get_path(tree_iter), open_all=False)
+    elif inst.nodes_bookm_exp and str(node_id) in inst.bookmarks:
+        inst.treeview_expand_to_tree_iter(tree_iter)
     tree_iter = treestore.iter_children(tree_iter)
     while tree_iter != None:
-        set_tree_expanded_collapsed_string_iter(tree_iter, expanded_collapsed_dict, treeview, treestore)
+        set_tree_expanded_collapsed_string_iter(inst, tree_iter, expanded_collapsed_dict, treeview, treestore)
         tree_iter = treestore.iter_next(tree_iter)
 
 def set_tree_path_and_cursor_pos(inst):
@@ -647,7 +659,7 @@ def preferences_tab_all_nodes(dad, vbox_all_nodes, pref_dialog):
 
     vbox_all_nodes.pack_start(frame_text_editor, expand=False)
     vbox_all_nodes.pack_start(frame_misc_all, expand=False)
-    
+
     def on_textbuffer_special_chars_changed(textbuffer, *args):
         new_special_chars = unicode(textbuffer.get_text(*textbuffer.get_bounds()).replace(cons.CHAR_NEWLINE, ""), cons.STR_UTF8, cons.STR_IGNORE)
         if dad.special_chars != new_special_chars:
@@ -762,6 +774,8 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
     hbox_misc_text.set_spacing(4)
     checkbutton_rt_show_white_spaces = gtk.CheckButton(_("Show White Spaces"))
     checkbutton_rt_show_white_spaces.set_active(dad.rt_show_white_spaces)
+    checkbutton_rt_highl_curr_line = gtk.CheckButton(_("Highlight Current Line"))
+    checkbutton_rt_highl_curr_line.set_active(dad.rt_highl_curr_line)
     checkbutton_codebox_auto_resize = gtk.CheckButton(_("Expand CodeBoxes Automatically"))
     checkbutton_codebox_auto_resize.set_active(dad.codebox_auto_resize)
     hbox_embfile_size = gtk.HBox()
@@ -781,6 +795,7 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
 
     vbox_misc_text = gtk.VBox()
     vbox_misc_text.pack_start(checkbutton_rt_show_white_spaces, expand=False)
+    vbox_misc_text.pack_start(checkbutton_rt_highl_curr_line, expand=False)
     vbox_misc_text.pack_start(checkbutton_codebox_auto_resize, expand=False)
     vbox_misc_text.pack_start(hbox_embfile_size, expand=False)
     vbox_misc_text.pack_start(hbox_misc_text, expand=False)
@@ -848,6 +863,11 @@ def preferences_tab_rich_text_nodes(dad, vbox_text_nodes, pref_dialog):
         if dad.syntax_highlighting == cons.RICH_TEXT_ID:
             dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.rt_show_white_spaces else 0)
     checkbutton_rt_show_white_spaces.connect('toggled', on_checkbutton_rt_show_white_spaces_toggled)
+    def on_checkbutton_rt_highl_curr_line_toggled(checkbutton):
+        dad.rt_highl_curr_line = checkbutton.get_active()
+        if dad.syntax_highlighting == cons.RICH_TEXT_ID:
+            dad.sourceview.set_highlight_current_line(dad.rt_highl_curr_line)
+    checkbutton_rt_highl_curr_line.connect('toggled', on_checkbutton_rt_highl_curr_line_toggled)
     def on_checkbutton_codebox_auto_resize_toggled(checkbutton):
         dad.codebox_auto_resize = checkbutton.get_active()
     checkbutton_codebox_auto_resize.connect('toggled', on_checkbutton_codebox_auto_resize_toggled)
@@ -880,14 +900,14 @@ def preferences_tab_plain_text_n_code_nodes(dad, vbox_code_nodes, pref_dialog):
     combobox_style_scheme.set_active_iter(dad.get_combobox_iter_from_value(dad.style_scheme_liststore, 0, dad.style_scheme))
     hbox_style_scheme.pack_start(label_style_scheme, expand=False)
     hbox_style_scheme.pack_start(combobox_style_scheme)
-    checkbutton_show_white_spaces = gtk.CheckButton(_("Show White Spaces"))
-    checkbutton_show_white_spaces.set_active(dad.show_white_spaces)
-    checkbutton_highlight_current_line = gtk.CheckButton(_("Highlight Current Line"))
-    checkbutton_highlight_current_line.set_active(dad.highl_curr_line)
+    checkbutton_pt_show_white_spaces = gtk.CheckButton(_("Show White Spaces"))
+    checkbutton_pt_show_white_spaces.set_active(dad.pt_show_white_spaces)
+    checkbutton_pt_highl_curr_line = gtk.CheckButton(_("Highlight Current Line"))
+    checkbutton_pt_highl_curr_line.set_active(dad.pt_highl_curr_line)
 
     vbox_syntax.pack_start(hbox_style_scheme, expand=False)
-    vbox_syntax.pack_start(checkbutton_show_white_spaces, expand=False)
-    vbox_syntax.pack_start(checkbutton_highlight_current_line, expand=False)
+    vbox_syntax.pack_start(checkbutton_pt_show_white_spaces, expand=False)
+    vbox_syntax.pack_start(checkbutton_pt_highl_curr_line, expand=False)
 
     frame_syntax = gtk.Frame(label="<b>"+_("Code Nodes")+"</b>")
     frame_syntax.get_label_widget().set_use_markup(True)
@@ -905,16 +925,16 @@ def preferences_tab_plain_text_n_code_nodes(dad, vbox_code_nodes, pref_dialog):
             dad.style_scheme = new_style
             support.dialog_info_after_restart(pref_dialog)
     combobox_style_scheme.connect('changed', on_combobox_style_scheme_changed)
-    def on_checkbutton_show_white_spaces_toggled(checkbutton):
-        dad.show_white_spaces = checkbutton.get_active()
+    def on_checkbutton_pt_show_white_spaces_toggled(checkbutton):
+        dad.pt_show_white_spaces = checkbutton.get_active()
         if dad.syntax_highlighting != cons.RICH_TEXT_ID:
-            dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.show_white_spaces else 0)
-    checkbutton_show_white_spaces.connect('toggled', on_checkbutton_show_white_spaces_toggled)
-    def on_checkbutton_highlight_current_line_toggled(checkbutton):
-        dad.highl_curr_line = checkbutton.get_active()
+            dad.sourceview.set_draw_spaces(codeboxes.DRAW_SPACES_FLAGS if dad.pt_show_white_spaces else 0)
+    checkbutton_pt_show_white_spaces.connect('toggled', on_checkbutton_pt_show_white_spaces_toggled)
+    def on_checkbutton_pt_highl_curr_line_toggled(checkbutton):
+        dad.pt_highl_curr_line = checkbutton.get_active()
         if dad.syntax_highlighting != cons.RICH_TEXT_ID:
-            dad.sourceview.set_highlight_current_line(dad.highl_curr_line)
-    checkbutton_highlight_current_line.connect('toggled', on_checkbutton_highlight_current_line_toggled)
+            dad.sourceview.set_highlight_current_line(dad.pt_highl_curr_line)
+    checkbutton_pt_highl_curr_line.connect('toggled', on_checkbutton_pt_highl_curr_line_toggled)
 
 def preferences_tab_tree(dad, vbox_tree, pref_dialog):
     """Preferences Dialog, Tree Tab"""
@@ -988,10 +1008,14 @@ def preferences_tab_tree(dad, vbox_tree, pref_dialog):
     radiobutton_nodes_startup_expand.set_group(radiobutton_nodes_startup_restore)
     radiobutton_nodes_startup_collapse = gtk.RadioButton(label=_("Collapse all Nodes"))
     radiobutton_nodes_startup_collapse.set_group(radiobutton_nodes_startup_restore)
+    checkbutton_nodes_bookm_exp = gtk.CheckButton(_("Nodes in Bookmarks Always Visible"))
+    checkbutton_nodes_bookm_exp.set_active(dad.nodes_bookm_exp)
+    checkbutton_nodes_bookm_exp.set_sensitive(dad.rest_exp_coll != 1)
 
     vbox_nodes_startup.pack_start(radiobutton_nodes_startup_restore, expand=False)
     vbox_nodes_startup.pack_start(radiobutton_nodes_startup_expand, expand=False)
     vbox_nodes_startup.pack_start(radiobutton_nodes_startup_collapse, expand=False)
+    vbox_nodes_startup.pack_start(checkbutton_nodes_bookm_exp, expand=False)
     frame_nodes_startup = gtk.Frame(label="<b>"+_("Nodes Status at Startup")+"</b>")
     frame_nodes_startup.get_label_widget().set_use_markup(True)
     frame_nodes_startup.set_shadow_type(gtk.SHADOW_NONE)
@@ -1079,14 +1103,23 @@ def preferences_tab_tree(dad, vbox_tree, pref_dialog):
         dad.treeview_refresh(change_icon=True)
     radiobutton_node_icon_none.connect('toggled', on_radiobutton_node_icon_none_toggled)
     def on_radiobutton_nodes_startup_restore_toggled(checkbutton):
-        if checkbutton.get_active(): dad.rest_exp_coll = 0
+        if checkbutton.get_active():
+            dad.rest_exp_coll = 0
+            checkbutton_nodes_bookm_exp.set_sensitive(True)
     radiobutton_nodes_startup_restore.connect('toggled', on_radiobutton_nodes_startup_restore_toggled)
     def on_radiobutton_nodes_startup_expand_toggled(checkbutton):
-        if checkbutton.get_active(): dad.rest_exp_coll = 1
+        if checkbutton.get_active():
+            dad.rest_exp_coll = 1
+            checkbutton_nodes_bookm_exp.set_sensitive(False)
     radiobutton_nodes_startup_expand.connect('toggled', on_radiobutton_nodes_startup_expand_toggled)
     def on_radiobutton_nodes_startup_collapse_toggled(checkbutton):
-        if checkbutton.get_active(): dad.rest_exp_coll = 2
+        if checkbutton.get_active():
+            dad.rest_exp_coll = 2
+            checkbutton_nodes_bookm_exp.set_sensitive(True)
     radiobutton_nodes_startup_collapse.connect('toggled', on_radiobutton_nodes_startup_collapse_toggled)
+    def on_checkbutton_nodes_bookm_exp_toggled(checkbutton):
+        dad.nodes_bookm_exp = checkbutton.get_active()
+    checkbutton_nodes_bookm_exp.connect('toggled', on_checkbutton_nodes_bookm_exp_toggled)
     def on_spinbutton_tree_nodes_names_width_value_changed(spinbutton):
         dad.cherry_wrap_width = int(spinbutton.get_value())
         dad.renderer_text.set_property('wrap-width', dad.cherry_wrap_width)
@@ -1148,13 +1181,13 @@ def preferences_tab_fonts(dad, vbox_fonts, pref_dialog):
     vbox_fonts.pack_start(frame_fonts, expand=False)
     def on_fontbutton_text_font_set(picker):
         dad.text_font = picker.get_font_name()
-        if dad.curr_tree_iter and dad.syntax_highlighting == cons.RICH_TEXT_ID:
+        if dad.curr_tree_iter and dad.syntax_highlighting in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]:
             dad.sourceview.modify_font(pango.FontDescription(dad.text_font))
     fontbutton_text.connect('font-set', on_fontbutton_text_font_set)
     def on_fontbutton_code_font_set(picker):
         dad.code_font = picker.get_font_name()
         if not dad.curr_tree_iter: return
-        if dad.syntax_highlighting != cons.RICH_TEXT_ID:
+        if dad.syntax_highlighting not in [cons.RICH_TEXT_ID, cons.PLAIN_TEXT_ID]:
             dad.sourceview.modify_font(pango.FontDescription(dad.code_font))
         else:
             support.rich_text_node_modify_codeboxes_font(dad.curr_buffer.get_start_iter(), dad.code_font)
@@ -1434,13 +1467,14 @@ def preferences_tab_misc(dad, vbox_misc, pref_dialog):
     adjustment_num_backups = gtk.Adjustment(value=dad.backup_num, lower=1, upper=100, step_incr=1)
     spinbutton_num_backups = gtk.SpinButton(adjustment_num_backups)
     spinbutton_num_backups.set_sensitive(dad.backup_copy)
+    spinbutton_num_backups.set_value(dad.backup_num)
     hbox_num_backups.pack_start(label_num_backups, expand=False)
     hbox_num_backups.pack_start(spinbutton_num_backups, expand=False)
     vbox_saving.pack_start(hbox_autosave, expand=False)
     vbox_saving.pack_start(checkbutton_autosave_on_quit, expand=False)
     vbox_saving.pack_start(checkbutton_backup_before_saving, expand=False)
     vbox_saving.pack_start(hbox_num_backups, expand=False)
-    
+
     checkbutton_autosave.set_active(dad.autosave[0])
     spinbutton_autosave.set_value(dad.autosave[1])
     spinbutton_autosave.set_sensitive(dad.autosave[0])
