@@ -2,7 +2,7 @@
 #
 #       machines.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -109,6 +109,10 @@ class XMLHandler:
                     self.append_tree_node(dom_iter, tree_father, discard_ids, node_sequence)
                 elif dom_iter.nodeName == "bookmarks":
                     self.dad.bookmarks = dom_iter.attributes['list'].value.split(",")
+                    for node_id_str in self.dad.bookmarks:
+                        tree_iter = self.dad.get_tree_iter_from_node_id(int(node_id_str))
+                        if tree_iter:
+                            self.dad.update_node_pre_icon(tree_iter, stock_id="pin")
                 dom_iter = dom_iter.nextSibling
             return True
 
@@ -154,7 +158,8 @@ class XMLHandler:
                                                             syntax_highlighting,
                                                             node_sequence,
                                                             node_tags,
-                                                            readonly])
+                                                            readonly,
+                                                            None])
         self.dad.nodes_names_dict[unique_id] = self.dad.treestore[tree_iter][1]
         if discard_ids:
             # we are importing nodes
@@ -524,13 +529,13 @@ class XMLHandler:
                 tag_names = [self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0])]
                 if not element[0]: continue
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (2+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 4 + 2*curr_node_level
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (4+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 6 + 2*curr_node_level
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), (6+2*curr_node_level)*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 8 + 2*curr_node_level
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -613,13 +618,13 @@ class XMLHandler:
                 tag_names = []
                 tag_names.append(self.dad.apply_tag_exist_or_create(tag_property, property_value + cons.CHAR_SPACE + element[0]))
                 if element[0][:2] == cons.TAG_PROP_H1:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 2
                 elif element[0][:2] == cons.TAG_PROP_H2:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 2*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 4
                 else:
-                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), 4*cons.CHAR_SPACE + cons.CHARS_LISTBUL[0] + cons.CHAR_SPACE)
                     curr_offset += 6
                 text_buffer.insert(text_buffer.get_iter_at_offset(curr_offset), cons.CHAR_NEWLINE)
                 text_buffer.insert_with_tags_by_name(text_buffer.get_iter_at_offset(curr_offset), element[1], *tag_names)
@@ -735,20 +740,25 @@ class StateMachine:
     def get_iter_alignment(self, iter_text):
         """Get the Alignment Value of the given Iter"""
         align_center = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_CENTER)
+        align_fill = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_FILL)
         align_right = self.dad.apply_tag_exist_or_create(cons.TAG_JUSTIFICATION, cons.TAG_PROP_RIGHT)
-        if iter_text.has_tag(self.dad.tag_table.lookup(align_center)): return cons.TAG_PROP_CENTER
-        elif iter_text.has_tag(self.dad.tag_table.lookup(align_right)): return cons.TAG_PROP_RIGHT
-        else: return cons.TAG_PROP_LEFT
+        if iter_text.has_tag(self.dad.tag_table.lookup(align_center)):
+            return cons.TAG_PROP_CENTER
+        if iter_text.has_tag(self.dad.tag_table.lookup(align_fill)):
+            return cons.TAG_PROP_FILL
+        if iter_text.has_tag(self.dad.tag_table.lookup(align_right)):
+            return cons.TAG_PROP_RIGHT
+        return cons.TAG_PROP_LEFT
 
     def load_embedded_image_element(self, text_buffer, element):
         """Load an Image from the Embedded Vector into the Buffer"""
         iter_insert = text_buffer.get_iter_at_offset(element[0])
-        self.dad.image_insert(iter_insert, element[1], element[2])
+        self.dad.image_insert(iter_insert, element[1], element[2], text_buffer=text_buffer)
 
     def load_embedded_table_element(self, text_buffer, element):
         """Load a Table from the Embedded Vector into the Buffer"""
         iter_insert = text_buffer.get_iter_at_offset(element[0])
-        self.dad.tables_handler.table_insert(iter_insert, element[1], element[2])
+        self.dad.tables_handler.table_insert(iter_insert, element[1], element[2], text_buffer=text_buffer)
 
     def load_embedded_codebox_element(self, text_buffer, element):
         """Load a CodeBox from the Embedded Vector into the Buffer"""
@@ -833,19 +843,19 @@ class StateMachine:
     def text_variation(self, node_id, varied_text):
         """Insertion or Removal of text in the given node_id"""
         if cons.CHAR_NEWLINE in varied_text:
-            self.update_state(node_id)
+            self.update_state()
             return
         alphanum = re.search("\w", varied_text, re.UNICODE) # we search for an alphanumeric character
         if self.nodes_indicators[node_id] < 2:
             if alphanum != None: self.nodes_indicators[node_id] = 2 # alphanumeric transition
             else: self.nodes_indicators[node_id] = 1 # non alphanumeric transition
         elif alphanum == None: # self.nodes_indicators[node_id] == 2 and non alphanumeric transition
-            self.update_state(node_id)
+            self.update_state()
 
     def requested_previous_state(self, node_id):
         """A Previous State, if Existing, is Requested"""
         if self.curr_index_is_last_index(node_id):
-            self.update_state(node_id)
+            self.update_state()
         if self.nodes_indexes[node_id] > 0:
             self.nodes_indexes[node_id] -= 1
         return self.nodes_vectors[node_id][self.nodes_indexes[node_id]]
@@ -875,8 +885,11 @@ class StateMachine:
         last_index = len(self.nodes_vectors[node_id]) - 1
         return curr_index == last_index
 
-    def update_state(self, node_id):
+    def update_state(self):
         """Update the state for the given node_id"""
+        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
+            return
+        node_id = self.dad.treestore[self.dad.curr_tree_iter][3]
         if not self.curr_index_is_last_index(node_id):
             del self.nodes_vectors[node_id][self.nodes_indexes[node_id]+1:]
         xml_content = self.dad.xml_handler.treestore_node_to_dom(self.dad.curr_tree_iter)
