@@ -2,7 +2,7 @@
 #
 #       main.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@ import gtk, gobject
 import dbus
 import dbus.service
 import dbus.mainloop.glib
-import sys, os, gettext
+import sys, os, subprocess, gettext
 import __builtin__
 msg_server_to_core = {'f':0, 'p':""}
 __builtin__.msg_server_to_core = msg_server_to_core
@@ -218,6 +218,11 @@ def main(args):
             object = CherryTreeObject(session_bus, '/CherryTreeObject')
             CherryTreeHandler(args, lang_str)
             gtk.main()
+            if cons.IS_WIN_OS:
+                si = subprocess.STARTUPINFO()
+                si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+                sys.stderr = os.devnull
+                subprocess.check_output(["taskkill", "/f", "/im", "dbus-daemon.exe"], startupinfo=si)
     else:
         print "dbus fail, maybe a firewall problem, centralized instances disabled"
         lang_str = initializations()
