@@ -2,7 +2,7 @@
 #
 #       tablez.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -367,7 +367,7 @@ class TablesHandler:
             self.dad.state_machine.apply_object_justification(text_iter, table_justification, text_buffer)
         elif self.dad.user_active:
             # if I apply a justification, the state is already updated
-            self.dad.state_machine.update_state(self.dad.treestore[self.dad.curr_tree_iter][3])
+            self.dad.state_machine.update_state()
 
     def table_edit_properties(self, *args):
         """Edit Table Properties"""
@@ -528,13 +528,13 @@ class TablesHandler:
             subseq_iter = model.iter_next(iter)
             if subseq_iter == None: return
             model.swap(iter, subseq_iter)
-        elif action == "sort_desc":
+        elif action == "sort_asc":
             father_iter = model.iter_parent(iter)
             movements = False
             while self.dad.node_siblings_sort_iteration(model, father_iter, True, 0):
                 movements = True
             if not movements: return
-        elif action == "sort_asc":
+        elif action == "sort_desc":
             father_iter = model.iter_parent(iter)
             movements = False
             while self.dad.node_siblings_sort_iteration(model, father_iter, False, 0):
